@@ -14,11 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A sink that generates an AVI file from a composite media session
 // Implementation
 
 #include "AVIFileSink.hh"
+#include "InputFile.hh"
 #include "OutputFile.hh"
 #include "GroupsockHelper.hh"
 
@@ -229,10 +230,15 @@ Boolean AVIFileSink::continuePlaying() {
 
 void AVIFileSink
 ::afterGettingFrame(void* clientData, unsigned packetDataSize,
-		    unsigned /*numTruncatedBytes*/,
+		    unsigned numTruncatedBytes,
 		    struct timeval presentationTime,
 		    unsigned /*durationInMicroseconds*/) {
   AVISubsessionIOState* ioState = (AVISubsessionIOState*)clientData;
+  if (numTruncatedBytes > 0) {
+    ioState->envir() << "AVIFileSink::afterGettingFrame(): The input frame data was too large for our buffer.  "
+		     << numTruncatedBytes
+		     << " bytes of trailing data was dropped!  Correct this by increasing the \"bufferSize\" parameter in the \"createNew()\" call.\n";
+  }
   ioState->afterGettingFrame(packetDataSize, presentationTime);
 }
 
@@ -471,7 +477,13 @@ void AVISubsessionIOState::useFrame(SubsessionBuffer& buffer) {
 
   // Write the data into the file:
   fOurSink.fNumBytesWritten += fOurSink.addWord(fAVISubsessionTag);
-  fOurSink.fNumBytesWritten += fOurSink.addWord(frameSize);
+  if (strcmp(fOurSubsession.codecName(), "H264") == 0) {
+    // Insert a 'start code' (0x00 0x00 0x00 0x01) in front of the frame:
+    fOurSink.fNumBytesWritten += fOurSink.addWord(4+frameSize);
+    fOurSink.fNumBytesWritten += fOurSink.addWord(fourChar(0x00, 0x00, 0x00, 0x01));//add start code
+  } else {
+    fOurSink.fNumBytesWritten += fOurSink.addWord(frameSize);
+  }
   fwrite(frameSource, 1, frameSize, fOurSink.fOutFid);
   fOurSink.fNumBytesWritten += frameSize;
   // Pad to an even length:
@@ -520,15 +532,15 @@ unsigned AVIFileSink::add4ByteString(char const* str) {
 
 void AVIFileSink::setWord(unsigned filePosn, unsigned size) {
   do {
-    if (fseek(fOutFid, filePosn, SEEK_SET) < 0) break;
+    if (SeekFile64(fOutFid, filePosn, SEEK_SET) < 0) break;
     addWord(size);
-    if (fseek(fOutFid, 0, SEEK_END) < 0) break; // go back to where we were
+    if (SeekFile64(fOutFid, 0, SEEK_END) < 0) break; // go back to where we were
 
     return;
   } while (0);
 
-  // One of the fseek()s failed, probable because we're not a seekable file
-  envir() << "AVIFileSink::setWord(): fseek failed (err "
+  // One of the SeekFile64()s failed, probable because we're not a seekable file
+  envir() << "AVIFileSink::setWord(): SeekFile64 failed (err "
 	  << envir().getErrno() << ")\n";
 }
 
@@ -537,7 +549,7 @@ void AVIFileSink::setWord(unsigned filePosn, unsigned size) {
 #define addFileHeader(tag,name) \
     unsigned AVIFileSink::addFileHeader_##name() { \
         add4ByteString("" #tag ""); \
-        unsigned headerSizePosn = ftell(fOutFid); addWord(0); \
+        unsigned headerSizePosn = (unsigned)TellFile64(fOutFid); addWord(0); \
         add4ByteString("" #name ""); \
         unsigned ignoredSize = 8;/*don't include size of tag or size fields*/ \
         unsigned size = 12
@@ -545,7 +557,7 @@ void AVIFileSink::setWord(unsigned filePosn, unsigned size) {
 #define addFileHeader1(name) \
     unsigned AVIFileSink::addFileHeader_##name() { \
         add4ByteString("" #name ""); \
-        unsigned headerSizePosn = ftell(fOutFid); addWord(0); \
+        unsigned headerSizePosn = (unsigned)TellFile64(fOutFid); addWord(0); \
         unsigned ignoredSize = 8;/*don't include size of name or size fields*/ \
         unsigned size = 8
 
@@ -602,11 +614,11 @@ addFileHeaderEnd;
 addFileHeader1(avih);
     unsigned usecPerFrame = fMovieFPS == 0 ? 0 : 1000000/fMovieFPS;
     size += addWord(usecPerFrame); // dwMicroSecPerFrame
-    fAVIHMaxBytesPerSecondPosition = ftell(fOutFid);
+    fAVIHMaxBytesPerSecondPosition = (unsigned)TellFile64(fOutFid);
     size += addWord(0); // dwMaxBytesPerSec (fill in later)
     size += addWord(0); // dwPaddingGranularity
     size += addWord(AVIF_TRUSTCKTYPE|AVIF_HASINDEX|AVIF_ISINTERLEAVED); // dwFlags
-    fAVIHFrameCountPosition = ftell(fOutFid);
+    fAVIHFrameCountPosition = (unsigned)TellFile64(fOutFid);
     size += addWord(0); // dwTotalFrames (fill in later)
     size += addWord(0); // dwInitialFrame
     size += addWord(fNumSubsessions); // dwStreams
@@ -634,7 +646,7 @@ addFileHeader1(strh);
     size += addWord(fCurrentIOState->fAVIScale); // dwScale
     size += addWord(fCurrentIOState->fAVIRate); // dwRate
     size += addWord(0); // dwStart
-    fCurrentIOState->fSTRHFrameCountPosition = ftell(fOutFid);
+    fCurrentIOState->fSTRHFrameCountPosition = (unsigned)TellFile64(fOutFid);
     size += addWord(0); // dwLength (fill in later)
     size += addWord(fBufferSize); // dwSuggestedBufferSize
     size += addWord((unsigned)-1); // dwQuality
