@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A simplified version of "H264VideoStreamFramer" that takes only complete,
 // discrete frames (rather than an arbitrary byte stream) as input.
 // This avoids the parsing and data copying overhead of the full
@@ -31,7 +31,7 @@ H264VideoStreamDiscreteFramer::createNew(UsageEnvironment& env, FramedSource* in
 
 H264VideoStreamDiscreteFramer
 ::H264VideoStreamDiscreteFramer(UsageEnvironment& env, FramedSource* inputSource)
-  : H264VideoStreamFramer(env, inputSource, False/*don't create a parser*/) {
+  : H264VideoStreamFramer(env, inputSource, False/*don't create a parser*/, False) {
 }
 
 H264VideoStreamDiscreteFramer::~H264VideoStreamDiscreteFramer() {
@@ -63,9 +63,14 @@ void H264VideoStreamDiscreteFramer
   u_int8_t nal_unit_type = frameSize == 0 ? 0xFF : fTo[0]&0x1F;
 
   // Check for a (likely) common error: NAL units that (erroneously) begin with a 0x00000001 or 0x000001 'start code'
-  //     (Those start codes should only be in byte-stream data; not data that consists of discrete NAL units.)
+  //     (Those start codes should only be in byte-stream data; *not* data that consists of discrete NAL units.)
+  //     Once again, to be clear: The NAL units that you feed to a "H264VideoStreamDiscreteFramer" MUST NOT include start codes.
   if (nal_unit_type == 0) {
-    envir() << "Warning: Invalid 'nal_unit_type': 0.  Does the NAL unit begin with a MPEG 'start code' by mistake?\n";
+    if (frameSize >= 4 && fTo[0] == 0 && fTo[1] == 0 && ((fTo[2] == 0 && fTo[3] == 1) || fTo[2] == 1)) {
+      envir() << "H264VideoStreamDiscreteFramer error: MPEG 'start code' seen in the input\n";
+    } else {
+      envir() << "Warning: Invalid 'nal_unit_type': 0\n";
+    }
   } else if (nal_unit_type == 7) { // Sequence parameter set (SPS)
     saveCopyOfSPS(fTo, frameSize);
   } else if (nal_unit_type == 8) { // Picture parameter set (PPS)
