@@ -183,7 +183,7 @@ void MultiFramedRTPSource::doGetNextFrame1() {
       fReorderingBuffer->releaseUsedPacket(nextPacket);
     }
 
-    if (fCurrentPacketCompletesFrame || fNumTruncatedBytes > 0) {
+    if (fCurrentPacketCompletesFrame) {
       // We have all the data that the client wants.
       if (fNumTruncatedBytes > 0) {
 	envir() << "MultiFramedRTPSource::doGetNextFrame1(): The total received frame size exceeds the client's buffer size ("
@@ -232,7 +232,12 @@ void MultiFramedRTPSource::networkReadHandler1() {
   Boolean readSuccess = False;
   do {
     Boolean packetReadWasIncomplete = fPacketReadInProgress != NULL;
-    if (!bPacket->fillInData(fRTPInterface, packetReadWasIncomplete)) break;
+    if (!bPacket->fillInData(fRTPInterface, packetReadWasIncomplete)) {
+      if (bPacket->bytesAvailable() == 0) {
+	envir() << "MultiFramedRTPSource error: Hit limit when reading incoming packet over TCP. Increase \"MAX_PACKET_SIZE\"\n";
+      }
+      break;
+    }
     if (packetReadWasIncomplete) {
       // We need additional read(s) before we can process the incoming packet:
       fPacketReadInProgress = bPacket;
@@ -322,7 +327,7 @@ void MultiFramedRTPSource::networkReadHandler1() {
 
 ////////// BufferedPacket and BufferedPacketFactory implementation /////
 
-#define MAX_PACKET_SIZE 10000
+#define MAX_PACKET_SIZE 20000
 
 BufferedPacket::BufferedPacket()
   : fPacketSize(MAX_PACKET_SIZE),
@@ -370,7 +375,9 @@ Boolean BufferedPacket::fillInData(RTPInterface& rtpInterface, Boolean& packetRe
 
   unsigned numBytesRead;
   struct sockaddr_in fromAddress;
-  if (!rtpInterface.handleRead(&fBuf[fTail], fPacketSize-fTail, numBytesRead, fromAddress, packetReadWasIncomplete)) {
+  unsigned const maxBytesToRead = bytesAvailable();
+  if (maxBytesToRead == 0) return False; // exceeded buffer size when reading over TCP
+  if (!rtpInterface.handleRead(&fBuf[fTail], maxBytesToRead, numBytesRead, fromAddress, packetReadWasIncomplete)) {
     return False;
   }
   fTail += numBytesRead;
@@ -418,7 +425,7 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
   getNextEnclosedFrameParameters(newFramePtr, fTail - fHead,
 				 frameSize, frameDurationInMicroseconds);
   if (frameSize > toSize) {
-    bytesTruncated = frameSize - toSize;
+    bytesTruncated += frameSize - toSize;
     bytesUsed = toSize;
   } else {
     bytesTruncated = 0;
@@ -580,12 +587,18 @@ BufferedPacket* ReorderingPacketBuffer
   // We're still waiting for our desired packet to arrive.  However, if
   // our time threshold has been exceeded, then forget it, and return
   // the head packet instead:
-  struct timeval timeNow;
-  gettimeofday(&timeNow, NULL);
-  unsigned uSecondsSinceReceived
-    = (timeNow.tv_sec - fHeadPacket->timeReceived().tv_sec)*1000000
-    + (timeNow.tv_usec - fHeadPacket->timeReceived().tv_usec);
-  if (uSecondsSinceReceived > fThresholdTime) {
+  Boolean timeThresholdHasBeenExceeded;
+  if (fThresholdTime == 0) {
+    timeThresholdHasBeenExceeded = True; // optimization
+  } else {
+    struct timeval timeNow;
+    gettimeofday(&timeNow, NULL);
+    unsigned uSecondsSinceReceived
+      = (timeNow.tv_sec - fHeadPacket->timeReceived().tv_sec)*1000000
+      + (timeNow.tv_usec - fHeadPacket->timeReceived().tv_usec);
+    timeThresholdHasBeenExceeded = uSecondsSinceReceived > fThresholdTime;
+  }
+  if (timeThresholdHasBeenExceeded) {
     fNextExpectedSeqNo = fHeadPacket->rtpSeqNo();
         // we've given up on earlier packets now
     packetLossPreceded = True;
