@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from an WAV audio file.
 // Implementation
@@ -45,7 +45,7 @@ WAVAudioFileServerMediaSubsession
 void WAVAudioFileServerMediaSubsession
 ::seekStreamSource(FramedSource* inputSource, double& seekNPT, double streamDuration, u_int64_t& numBytes) {
   WAVAudioFileSource* wavSource;
-  if (fBitsPerSample == 16) {
+  if (fBitsPerSample > 8) {
     // "inputSource" is a filter; its input source is the original WAV file source:
     wavSource = (WAVAudioFileSource*)(((FramedFilter*)inputSource)->inputSource());
   } else {
@@ -54,10 +54,10 @@ void WAVAudioFileServerMediaSubsession
   }
 
   unsigned seekSampleNumber = (unsigned)(seekNPT*fSamplingFrequency);
-  unsigned seekByteNumber = (seekSampleNumber*fNumChannels*fBitsPerSample)/8;
+  unsigned seekByteNumber = seekSampleNumber*((fNumChannels*fBitsPerSample)/8);
 
   unsigned numDurationSamples = (unsigned)(streamDuration*fSamplingFrequency);
-  unsigned numDurationBytes = (numDurationSamples*fNumChannels*fBitsPerSample)/8;
+  unsigned numDurationBytes = numDurationSamples*((fNumChannels*fBitsPerSample)/8);
   numBytes = (u_int64_t)numDurationBytes;
 
   wavSource->seekToPCMByte(seekByteNumber, numDurationBytes);
@@ -67,7 +67,7 @@ void WAVAudioFileServerMediaSubsession
 ::setStreamSourceScale(FramedSource* inputSource, float scale) {
   int iScale = (int)scale;
   WAVAudioFileSource* wavSource;
-  if (fBitsPerSample == 16) {
+  if (fBitsPerSample > 8) {
     // "inputSource" is a filter; its input source is the original WAV file source:
     wavSource = (WAVAudioFileSource*)(((FramedFilter*)inputSource)->inputSource());
   } else {
@@ -82,42 +82,41 @@ FramedSource* WAVAudioFileServerMediaSubsession
 ::createNewStreamSource(unsigned /*clientSessionId*/, unsigned& estBitrate) {
   FramedSource* resultSource = NULL;
   do {
-    WAVAudioFileSource* wavSource
-      = WAVAudioFileSource::createNew(envir(), fFileName);
+    WAVAudioFileSource* wavSource = WAVAudioFileSource::createNew(envir(), fFileName);
     if (wavSource == NULL) break;
 
     // Get attributes of the audio source:
 
     fAudioFormat = wavSource->getAudioFormat();
     fBitsPerSample = wavSource->bitsPerSample();
-    if (!(fBitsPerSample == 4 || fBitsPerSample == 8 || fBitsPerSample == 16)) {
+    // We handle only 4,8,16,20,24 bits-per-sample audio:
+    if (fBitsPerSample%4 != 0 || fBitsPerSample < 4 || fBitsPerSample > 24 || fBitsPerSample == 12) {
       envir() << "The input file contains " << fBitsPerSample << " bit-per-sample audio, which we don't handle\n";
       break;
     }
     fSamplingFrequency = wavSource->samplingFrequency();
     fNumChannels = wavSource->numChannels();
-    unsigned bitsPerSecond
-      = fSamplingFrequency*fBitsPerSample*fNumChannels;
+    unsigned bitsPerSecond = fSamplingFrequency*fBitsPerSample*fNumChannels;
 
-    fFileDuration = (float)((8.0*wavSource->numPCMBytes())
-      /(fSamplingFrequency*fNumChannels*fBitsPerSample));
+    fFileDuration = (float)((8.0*wavSource->numPCMBytes())/(fSamplingFrequency*fNumChannels*fBitsPerSample));
 
     // Add in any filter necessary to transform the data prior to streaming:
-    if (fBitsPerSample == 16) {
-      // Note that samples in the WAV audio file are in little-endian order.
-      if (fConvertToULaw) {
-	// Add a filter that converts from raw 16-bit PCM audio
-	// to 8-bit u-law audio:
-	resultSource
-	  = uLawFromPCMAudioSource::createNew(envir(), wavSource, 1/*little-endian*/);
-	bitsPerSecond /= 2;
-      } else {
+    resultSource = wavSource; // by default
+    if (fAudioFormat == WA_PCM) {
+      if (fBitsPerSample == 16) {
+	// Note that samples in the WAV audio file are in little-endian order.
+	if (fConvertToULaw) {
+	  // Add a filter that converts from raw 16-bit PCM audio to 8-bit u-law audio:
+	  resultSource = uLawFromPCMAudioSource::createNew(envir(), wavSource, 1/*little-endian*/);
+	  bitsPerSecond /= 2;
+	} else {
+	  // Add a filter that converts from little-endian to network (big-endian) order:
+	  resultSource = EndianSwap16::createNew(envir(), wavSource);
+	}
+      } else if (fBitsPerSample == 20 || fBitsPerSample == 24) {
 	// Add a filter that converts from little-endian to network (big-endian) order:
-	resultSource = EndianSwap16::createNew(envir(), wavSource);
+	resultSource = EndianSwap24::createNew(envir(), wavSource);
       }
-    } else { // fBitsPerSample == 8
-      // Don't do any transformation; send the 8-bit PCM data 'as is':
-      resultSource = wavSource;
     }
 
     estBitrate = (bitsPerSecond+500)/1000; // kbps
@@ -135,15 +134,13 @@ RTPSink* WAVAudioFileServerMediaSubsession
 		   FramedSource* /*inputSource*/) {
   do {
     char const* mimeType;
-    unsigned char payloadFormatCode;
+    unsigned char payloadFormatCode = rtpPayloadTypeIfDynamic; // by default, unless a static RTP payload type can be used
     if (fAudioFormat == WA_PCM) {
       if (fBitsPerSample == 16) {
 	if (fConvertToULaw) {
 	  mimeType = "PCMU";
 	  if (fSamplingFrequency == 8000 && fNumChannels == 1) {
 	    payloadFormatCode = 0; // a static RTP payload type
-	  } else {
-	    payloadFormatCode = rtpPayloadTypeIfDynamic;
 	  }
 	} else {
 	  mimeType = "L16";
@@ -151,31 +148,27 @@ RTPSink* WAVAudioFileServerMediaSubsession
 	    payloadFormatCode = 10; // a static RTP payload type
 	  } else if (fSamplingFrequency == 44100 && fNumChannels == 1) {
 	    payloadFormatCode = 11; // a static RTP payload type
-	  } else {
-	    payloadFormatCode = rtpPayloadTypeIfDynamic;
 	  }
 	}
-      } else { // fBitsPerSample == 8
+      } else if (fBitsPerSample == 20) {
+	mimeType = "L20";
+      } else if (fBitsPerSample == 24) {
+	mimeType = "L24";
+      } else { // fBitsPerSample == 8 (we assume that fBitsPerSample == 4 is only for WA_IMA_ADPCM)
 	mimeType = "L8";
-	payloadFormatCode = rtpPayloadTypeIfDynamic;
       }
     } else if (fAudioFormat == WA_PCMU) {
       mimeType = "PCMU";
       if (fSamplingFrequency == 8000 && fNumChannels == 1) {
 	payloadFormatCode = 0; // a static RTP payload type
-      } else {
-	payloadFormatCode = rtpPayloadTypeIfDynamic;
       }
     } else if (fAudioFormat == WA_PCMA) {
       mimeType = "PCMA";
       if (fSamplingFrequency == 8000 && fNumChannels == 1) {
 	payloadFormatCode = 8; // a static RTP payload type
-      } else {
-	payloadFormatCode = rtpPayloadTypeIfDynamic;
       }
     } else if (fAudioFormat == WA_IMA_ADPCM) {
       mimeType = "DVI4";
-      payloadFormatCode = rtpPayloadTypeIfDynamic; // by default; could be changed below:
       // Use a static payload type, if one is defined:
       if (fNumChannels == 1) {
 	if (fSamplingFrequency == 8000) {
@@ -189,8 +182,8 @@ RTPSink* WAVAudioFileServerMediaSubsession
 	}
       }
     } else { //unknown format
-		break;
-	}
+      break;
+    }
 
     return SimpleRTPSink::createNew(envir(), rtpGroupsock,
 				    payloadFormatCode, fSamplingFrequency,
