@@ -16,6 +16,10 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Copyright (c) 1996-2012, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
+//
+// NOTE: If you want to develop your own RTSP client application (or embed RTSP client functionality into your own application),
+// then we don't recommend using this code as a model, because it is too complex (with many options).
+// Instead, we recommend using the "testRTSPClient" application code as a model.
 
 #include "playCommon.hh"
 #include "BasicUsageEnvironment.hh"
@@ -70,6 +74,7 @@ int verbosityLevel = 1; // by default, print verbose output
 double duration = 0;
 double durationSlop = -1.0; // extra seconds to play at the end
 double initialSeekTime = 0.0f;
+char* initialAbsoluteSeekTime = NULL;
 float scale = 1.0f;
 double endTime;
 unsigned interPacketGapMaxTime = 0;
@@ -112,7 +117,7 @@ void usage() {
        << " [-u <username> <password>"
 	   << (allowProxyServers ? " [<proxy-server> [<proxy-server-port>]]" : "")
        << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-M <mime-subtype-name>]" : "")
-       << " [-s <initial-seek-time>] [-z <scale>]"
+       << " [-s <initial-seek-time>]|[-U <absolute-seek-time>] [-z <scale>]"
        << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-I <input-interface-ip-address>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
   shutdown();
 }
@@ -293,6 +298,7 @@ int main(int argc, char** argv) {
     }
 
     case 'u': { // specify a username and password
+      if (argc < 4) usage(); // there's no argv[3] (for the "password")
       username = argv[2];
       password = argv[3];
       argv+=2; argc-=2;
@@ -422,6 +428,13 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'U': {
+      // specify initial absolute seek time (trick play), using a string of the form "YYYYMMDDTHHMMSSZ" or "YYYYMMDDTHHMMSS.<frac>Z
+      initialAbsoluteSeekTime = argv[2];
+      ++argv; --argc;
+      break;
+    }
+
     case 'z': { // scale (trick play)
       float arg;
       if (sscanf(argv[2], "%g", &arg) != 1 || arg == 0.0f) {
@@ -440,14 +453,14 @@ int main(int argc, char** argv) {
 
     ++argv; --argc;
   }
-  if (argc != 2) usage();
+  if (argc != 2) usage(); // there must be exactly one "rtsp://" URL at the end
   if (outputQuickTimeFile && outputAVIFile) {
-    *env << "The -i and -q (or -4) flags cannot both be used!\n";
+    *env << "The -i and -q (or -4) options cannot both be used!\n";
     usage();
   }
   Boolean outputCompositeFile = outputQuickTimeFile || outputAVIFile;
   if (!createReceivers && outputCompositeFile) {
-    *env << "The -r and -q (or -4 or -i) flags cannot both be used!\n";
+    *env << "The -r and -q (or -4 or -i) options cannot both be used!\n";
     usage();
   }
   if (outputCompositeFile && !movieWidthOptionSet) {
@@ -463,16 +476,20 @@ int main(int argc, char** argv) {
 	 << movieFPS << " frames-per-second\n";
   }
   if (audioOnly && videoOnly) {
-    *env << "The -a and -v flags cannot both be used!\n";
+    *env << "The -a and -v options cannot both be used!\n";
     usage();
   }
   if (sendOptionsRequestOnly && !sendOptionsRequest) {
-    *env << "The -o and -O flags cannot both be used!\n";
+    *env << "The -o and -O options cannot both be used!\n";
+    usage();
+  }
+  if (initialAbsoluteSeekTime != NULL && initialSeekTime != 0.0f) {
+    *env << "The -s and -U options cannot both be used!\n";
     usage();
   }
   if (tunnelOverHTTPPortNum > 0) {
     if (streamUsingTCP) {
-      *env << "The -t and -T flags cannot both be used!\n";
+      *env << "The -t and -T options cannot both be used!\n";
       usage();
     } else {
       streamUsingTCP = True;
@@ -791,7 +808,14 @@ void setupStreams() {
     if (endTime < 0) endTime = 0.0f;
   }
 
-  startPlayingSession(session, initialSeekTime, endTime, scale, continueAfterPLAY);
+  char const* absStartTime = initialAbsoluteSeekTime != NULL ? initialAbsoluteSeekTime : session->absStartTime();
+  if (absStartTime != NULL) {
+    // Either we or the server have specified that seeking should be done by 'absolute' time:
+    startPlayingSession(session, absStartTime, session->absEndTime(), scale, continueAfterPLAY);
+  } else {
+    // Normal case: Seek by relative time (NPT):
+    startPlayingSession(session, initialSeekTime, endTime, scale, continueAfterPLAY);
+  }
 }
 
 void continueAfterPLAY(RTSPClient*, int resultCode, char* resultString) {
